-- =======================================================================
--                             Along The River
-- =======================================================================

-- Checks objective for done flag.
function obj_done(objective)
	local done = false
	if objective then
		done = objective.done
	end
	return done
end

-- =======================================================================

-- Places building at field if player owns field and extra field for flag.
function place_building_if_owner(player, building, point, cs)
	local build = false
	if point.owner == player and point.brn.owner == player then
		player:place_building(building, point, cs, true)
		build = true
	end
	return build
end

-- =======================================================================

function remove_type(point, _type)
	local immovable = point.immovable
	if immovable then
		if immovable.type == _type then
			immovable:remove()
		end
	end
end

-- =======================================================================

function immovable_has_name(immovable, name)
	local valid = false
	if immovable then
		if immovable.name == name then valid = true end
	end
	return valid
end

-- =======================================================================

function add_wares(point, ware, amount)
	local immovable = point.immovable
	if immovable then
		if immovable.type == "flag" or immovable.type == "warehouse" or immovable.type == "productionsite" then
			immovable:set_wares(ware, immovable:get_wares(ware) + amount)
		end
	end
end

-- =======================================================================

-- Transports wares to flag, warehouse or productionsite.
-- Waits if the building does not exist.
function transport_wares(player, point, ware, amount, alert, image)
	local count = amount
	while count > 0 do
		sleep(499)
		local immovable = point.immovable
		if immovable then
			if immovable.type == "flag" or immovable.type == "warehouse" or immovable.type == "productionsite" then
				immovable:set_wares(ware, immovable:get_wares(ware) + 1)
				count = count - 1
			end
		end
	end
	if(alert) then
		local immovable = point.immovable
		if immovable then
			if immovable.type == "flag" or immovable.type == "warehouse" or immovable.type == "productionsite" then
				if image then
					player:send_message(_"Transport", "<rt image="..image.." image-align=left><p line-spacing=0 font-size=12>"..amount.._" units of "..ware.." have been brought to your "..immovable.descname..".</p></rt>", { field = point })
				else
					player:send_message(_"Transport", amount.._" units of "..ware.._" have been brought to your "..immovable.descname..".", { field = point })
				end
			end
		end
	end
end

-- =======================================================================

-- Spawns soldiers in a warehouse, militarysite or trainingsite with intervals in ms.
-- e.g.: spawn_soldiers(map:get_field(hq_x,hq_y), {3,5,0,2}, 45, 4999)
-- Stops if the object does not exist or is invalid.
function spawn_soldiers(point, ranking, count, interval)
	while count > 0 do
		sleep(interval)
		local immovable = point.immovable
		if immovable then
			if immovable.type == "warehouse" or immovable.type == "militarysite" or immovable.type == "trainingsite" then
				immovable:set_soldiers(ranking, immovable:get_soldiers(ranking) + 1)
			else
				break
			end
		else
			break
		end
		count = count - 1
	end
end

-- =======================================================================

-- Checks if enough wares are available at flag, warehouse or productionsite and removes if true.
-- Returns true if wares are removed.
function wares_collected_at_field(point, ware, amount)
	local valid = false
	local immovable = point.immovable
	if immovable then
		if immovable.type == "flag" or immovable.type == "warehouse" or immovable.type == "productionsite" then
			if immovable:get_wares(ware) >= amount then
				immovable:set_wares(ware, immovable:get_wares(ware) - amount)
				valid = true
			end
		end
	end
	return valid
end

-- =======================================================================

-- Lists all tribe related names.
function list_names(player, headquarters)
	local wares = headquarters.immovable:get_wares("all")
	for k in pairs(wares) do print(k) end

	local workers = headquarters.immovable:get_workers("all")
	for k in pairs(workers) do print(k) end

	local building = player.allowed_buildings
	for k in pairs(building) do print(k) end
end

-- =======================================================================

-- Removes whitespace at the beginning or the end and translates
-- newlines into spaces.
function reflow(s)
   s = s:gsub("\r", ""):gsub("\n", " ")
   return s:match'^%s*(.*%S)' or ''
end

-- =======================================================================

-- Formats message box with objectives.
function _msg(title, text, ...)
	local s = "<rt text-align=center><p line-spacing=8 font-size=20 font-face=DejaVuSerif font-weight=bold font-color=EEBB00>"..title.."</p></rt><rt image=map:advisor.png image-align=right><p line-spacing=0 font-size=12>"..reflow(text)
	objs = {...}
	if #objs > 0 then
		s = s.."<br><br>".._"-- NEW OBJECTIVES --".."<br>"
		local i = 1
		while i < #objs do
			s = s..reflow(objs[i].body).."<br>"
			i = i + 1
		end
		s = s..reflow(objs[#objs].body)
	end
	return s.."</p></rt>"
end

-- =======================================================================

function send_msg(t)
	p1:message_box(t.title, t.body, t)
end

-- =======================================================================

function _obj(t, image)
	if image then
		return "<rt image="..image.." image-align=left><p line-spacing=0 font-size=12>"..reflow(t).."</p></rt>"
	else
		return "<rt><p line-spacing=0 font-size=12>"..reflow(t).."</p></rt>"
	end
end

-- =======================================================================

function add_obj(o)
	return p1:add_objective(o.name, o.title, _obj(o.body, o.image))
end

-- =======================================================================
